--[[
FISHING SKILL TRACKER FOR TITAN PANEL

Plug-in for Titan Panel to display the current "buffed" fishing skill
of the player.

Maintained by: John McAllister (jandd661)
Author: John McAllister (jandd661)

NOTE: There are many things I don't like about the code here so I will be
cleaning it up in future releases.

Copyright 2015 - 2016 John McAllister
--------------------------------------------------------------------------
This file is part of Titan Fishing Skill Tracker.

Titan Fishing Skill Tracker is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Titan Fishing Skill Tracker is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License (Licence.txt)
along with Titan Fishing Skill Tracker.  If not, see <http://www.gnu.org/licenses/>.

You can contact jand661 by PM at CurseForge.com
http://wow.curseforge.com/addons/titan-fishing-skill-tracker/
--------------------------------------------------------------------------
]]--
local FST_LuaVersion = "r003"
local FST_JD661_UTILS_VERSION_REQ = "r003"
local FST_AddonName = "TitanFishingSkillTracker"
local FST_IconPath = "Interface\\Icons\\Trade_fishing.blp"
local FST_MenuText = "Fishing Skill Tracker"
local FST_ButtonLabel = "Fishing Skill: "
local FST_ToolTipTitle = "Fishing Skill Info"
local FST_ID = "FST"
local CURRENT_MAX_SKILL = 800
local strRightClickHideMenuText = "Hide Fishing Skill Tracker"
local i = 0
local intCurrentSkillLevel = 0
local prevFST = 0
local FSTmaxtrained = 0
local FSTIncrease = 0
local FSTtextForTooltip
local strTextForTraining = " YOU NEED TO TRAIN!"
local intForceNeedTraining = 0
------------------------------------------------------
 -- Only update the button every n cycles of self:SetScript("OnUpdate", FST_Main)
 -- 10 cycles seems to be about a second.
------------------------------------------------------
local intUpdateOnCycle = 10
------------------------------------------------------
-- Set intDisplayDebugInfo to 1 
-- for some debug info in the tool tip text
-- assuming it makes it that far lol.
-- Not very useful at the moment
------------------------------------------------------
strDebugInfo = ""
intDisplayDebugInfo = 0
------------------------------------------------------
-- Create a Titan button 
------------------------------------------------------
local FishingFrame = CreateFrame("Button", "TitanPanelFSTButton", CreateFrame("Frame", nil, UIParent), "TitanPanelComboTemplate")
	FishingFrame:SetFrameStrata("FULLSCREEN")
	FishingFrame:SetScript("OnEvent", function(this, event, ...) this[event](this, ...) end)
	FishingFrame:RegisterEvent("ADDON_LOADED")
	--FishingFrame:RegisterEvent("UNIT_AURA")-- FOR LATER USE
	--FishingFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")-- FOR LATER USE
	--FishingFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")-- FOR LATER USE
	

--[[
-- FOR LATER USE
function FishingFrame:UNIT_SPELLCAST_CHANNEL_START(UnitID,SpellName,SpellRank,SpellLineIDCounter, SpellID)
	FST_UnitSpellCastChannelStart(UnitID,SpellName,SpellRank,SpellLineIDCounter, SpellID)
end

function FishingFrame:UNIT_SPELLCAST_CHANNEL_STOP(UnitID,SpellName,SpellRank,SpellLineIDCounter, SpellID)
	FST_UnitSpellCastChannelStop(UnitID,SpellName,SpellRank,SpellLineIDCounter, SpellID)
end

function FishingFrame:UNIT_AURA(UnitID)
	FST_UnitAura(UnitID)
end
]]--

---------------------------------------------
--
---------------------------------------------
function FishingFrame:ADDON_LOADED(thisAddon)
	if intDisplayDebugInfo == 1 then
		print ("FST Debug ADDON_LOADED:" .. thisAddon)
	end
	if thisAddon ~= FST_AddonName then 
		return 
	end
	local strTocVersion = GetAddOnMetadata(FST_AddonName, "X-Version")
	local strLuaVersion = FST_LuaVersion
	if(FST_JD661_UTILS_VERSION_REQ ~= JD661_UTILS_VERSION) then
		print("Titan Fishing Skill Tracker: Utils version: " .. JD661_UTILS_VERSION .. " not matching Lua required version: " .. FST_JD661_UTILS_VERSION_REQ ..". This addon may not work correctly")	
	end
	if(strTocVersion ~= strLuaVersion) then
		print("Titan Fishing Skill Tracker: Toc version: " .. strTocVersion .. " not matching Lua version: " .. strLuaVersion ..". This addon may not work correctly")
	end
	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
	self.registry = {
		id = FST_ID,
		menuText = FST_MenuText .. JD661Utils_ColorText(" (" .. strTocVersion .. ")","green"),
		buttonTextFunction = "TitanPanelFSTButton_GetButtonText",
		tooltipTitle = FST_ToolTipTitle,
		tooltipTextFunction = "TitanPanelFSTButton_GetTooltipText",
		frequency = 1,
		icon = FST_IconPath,
		iconWidth = 16,
		category = "Profession",
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = false,
			
		},
	}
	self:SetScript("OnUpdate", FST_Update)
		
end
-------------------------------------------
-- Used to meter the auto update of the button from
-- self:SetScript("OnUpdate", FST_Main)
-- There is no need to start a bunch of math
-- and UI calls every millisecond. 
-------------------------------------------
function FST_Update()
	--print(i)
	if(i > intUpdateOnCycle) then
		FST_Main()
		i = 0
	else
		i = i + 1
	end
end
---------------------------------------------
-- The guts of the addon
---------------------------------------------
function FST_Main()
	--local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(index)
	local profession_1, profession_2, archaeology, fishing, cooking, firstAid = GetProfessions();

	if fishing ~= nil then
		local name, _, skillLevel, maxSkillLevel, _, _, _, IncreaseSkillLevel = GetProfessionInfo(fishing)
		intCurrentSkillLevel = skillLevel
		FSTmaxtrained = maxSkillLevel
		FSTIncrease = IncreaseSkillLevel
		
		if intCurrentSkillLevel == prevFST and prevFSTmaxtrained == FSTmaxtrained and prevFSTIncrease == FSTIncrease then
			--------------------------------------
			-- Nothing has changed - do nothing
			--------------------------------------
		else
			-----------------------------------
			-- Got this far so something changed
			-----------------------------------
			prevFSTmaxtrained = FSTmaxtrained
			prevFST = intCurrentSkillLevel
			prevFSTIncrease = FSTIncrease
			TitanPanelButton_UpdateButton(FST_ID)
		end
	end
end
----------------------------------------------
-- Create the text for the Titan Bar button
----------------------------------------------
function TitanPanelFSTButton_GetButtonText()
	local FSTtext
	local strIncrease
	local intIsAtMaxTrained
	local strTextForTooltip
	
	intIsAtMaxTrained = 0	
	strIncrease = ""
	strTextForTooltip = ""

	if not intCurrentSkillLevel then
		FSTtext = "??"
	else
		----------------------------
		-- Any fishing bonuses?
		----------------------------
		if FSTIncrease > 0 then
			------------------------------------------
			-- Yes, Make the current skill text green
			------------------------------------------
			strIncrease = JD661Utils_ColorText((intCurrentSkillLevel + FSTIncrease),"green") 
			FSTtext = strIncrease .. "/" .. FSTmaxtrained
		else
			-------------------------------------------
			-- No, just normal text
			-------------------------------------------
			FSTtext = intCurrentSkillLevel .. "/" .. FSTmaxtrained 
		end
		------------------------------------------
		-- Are they at max trained skill?
		------------------------------------------
		if intCurrentSkillLevel >= FSTmaxtrained then
			intIsAtMaxTrained = 1 
			---------------------------------------
			-- YES, only display their max skill with bonuses
			---------------------------------------
			if FSTIncrease > 0 then
				strIncrease = intCurrentSkillLevel + FSTIncrease
				FSTtext = JD661Utils_ColorText(strIncrease,"green")
			else
				strIncrease = intCurrentSkillLevel
				FSTtext = intCurrentSkillLevel 
			end
		----------------------------------------
		-- NO, display their current skill with bonuses and what their max trained skill is
		----------------------------------------
		else
			intIsAtMaxTrained = 0 
			if FSTIncrease > 0 then
				strIncrease = intCurrentSkillLevel + FSTIncrease
				FSTtext = JD661Utils_ColorText(strIncrease,"green") .. "/" .. FSTmaxtrained
			else
				strIncrease = intCurrentSkillLevel
				FSTtext = intCurrentSkillLevel .. "/" .. FSTmaxtrained 
			end
		end
		strTextForTooltip = "Current Skill:\t" .. intCurrentSkillLevel .. "\nBonus Skill:\t" .. JD661Utils_ColorText(FSTIncrease,"green") .. "\nActual Skill:\t" .. strIncrease
		----------------------------------------
		-- If they need to train, let them know
		----------------------------------------
		if CURRENT_MAX_SKILL > FSTmaxtrained and FSTmaxtrained == intCurrentSkillLevel then
			strTextForTooltip = strTextForTooltip .. "\n\n" .. JD661Utils_ColorText(strTextForTraining,"red")
		else
			-----------------------------------
			-- Force text for testing/formatting 
			-----------------------------------
			if intForceNeedTraining == 1 then 
				strTextForTooltip = strTextForTooltip .. "\n\n" .. JD661Utils_ColorText(strTextForTraining,"red")
			end
		end
	end
	----------------------------------
	-- Add debug info
	----------------------------------
	if intDisplayDebugInfo == 1 then
		FSTtextForTooltip = strTextForTooltip
	else
		FSTtextForTooltip = strTextForTooltip
	end

	return FST_ButtonLabel, FSTtext
end
-----------------------------------------------
-- Set the tooltip text
-----------------------------------------------
function TitanPanelFSTButton_GetTooltipText()
	if FSTtextForTooltip then
		return FSTtextForTooltip
	else
		return "Displays your current fishing Skill level"
	end
end
----------------------------------------------------
-- What to do with a right-click on the button
----------------------------------------------------
function TitanPanelRightClickMenu_PrepareFSTMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[FST_ID].menuText)
	TitanPanelRightClickMenu_AddToggleIcon(FST_ID)
	TitanPanelRightClickMenu_AddToggleLabelText(FST_ID)
	TitanPanelRightClickMenu_AddSpacer()
	TitanPanelRightClickMenu_AddCommand(strRightClickHideMenuText, FST_ID, TITAN_PANEL_MENU_FUNC_HIDE)
end
